package tech.nevets.codebadgerestapi;

import com.google.gson.Gson;
import com.google.gson.JsonObject;
import org.apache.batik.dom.GenericDOMImplementation;
import org.apache.batik.svggen.SVGGraphics2D;
import org.apache.batik.svggen.SVGGraphics2DIOException;
import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;

import java.awt.*;
import java.io.*;
import java.net.URI;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;

import java.util.HashMap;
import java.util.Map;

public class BadgeFactory {

    public static byte[] genSVGBadge(String leftStr, String rightStr, Color secondaryColor, boolean isRounded) {
        DOMImplementation domImpl = GenericDOMImplementation.getDOMImplementation();
        Document doc = domImpl.createDocument("http://www.w3.org/2000/svg", "svg", null);
        SVGGraphics2D graphics2D = new SVGGraphics2D(doc);
        Font font = new Font("Calibri", Font.PLAIN, 16);

        graphics2D.setFont(font);
        FontMetrics fm = graphics2D.getFontMetrics();
        int lWidth = fm.stringWidth(leftStr);
        int rWidth = fm.stringWidth(rightStr);
        int totalWidth = lWidth + rWidth + 40;
        int totalHeight = 20;

        graphics2D.setSVGCanvasSize(new Dimension(totalWidth, totalHeight));

        graphics2D.setPaint(Color.GRAY);
        if (isRounded) {
            graphics2D.fillRoundRect(0, 0, totalWidth, totalHeight, 10, 10);
        } else {
            graphics2D.fillRect(0, 0, totalWidth, totalHeight);
        }
        graphics2D.setPaint(secondaryColor);
        if (isRounded) {
            graphics2D.fillRoundRect(totalWidth - rWidth - 20, 0, rWidth + 20, totalHeight, 10, 10);
        } else {
            graphics2D.fillRect(totalWidth - rWidth - 20, 0, rWidth + 20, totalHeight);
        }
        graphics2D.setPaint(Color.GRAY);
        graphics2D.drawString(leftStr, 10, (totalHeight / 2) + 6);
        graphics2D.drawString(rightStr, totalWidth - rWidth - 10, (totalHeight / 2) + 6);
        graphics2D.setPaint(Color.WHITE);
        graphics2D.drawString(leftStr, 10, (totalHeight / 2) + 5);
        graphics2D.drawString(rightStr, totalWidth - rWidth - 10, (totalHeight / 2) + 5);

        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        OutputStreamWriter osw = new OutputStreamWriter(baos);
        try {
            graphics2D.stream(osw, true);
        } catch (SVGGraphics2DIOException e) {
            e.printStackTrace();
            return new byte[]{};
        }
        return baos.toByteArray();
    }

    public static String getVersionFromGitBranch(String gitUrl, String org, String repo, String branch) {
        String versionSourcesUrlBase;
        if (org == null && repo == null) {
            if (gitUrl.endsWith(".git")) {
                versionSourcesUrlBase = gitUrl.substring(0, gitUrl.length() - 4) + "/raw/branch/" + branch;
            } else {
                versionSourcesUrlBase = gitUrl + "/raw/branch/" + branch;
            }
            if (!gitUrl.startsWith("https://")) {
                versionSourcesUrlBase = "https://" + versionSourcesUrlBase;
            }
        } else {
            versionSourcesUrlBase = "https://" + gitUrl + "/" + org + "/" + repo + "/raw/branch/" + branch;
        }
        Map<Source, String> VERSION_SOURCES = new HashMap<>();
        VERSION_SOURCES.put(Source.GRADLE_GROOVY, versionSourcesUrlBase + "/build.gradle");
        VERSION_SOURCES.put(Source.GRADLE_KOTLIN,versionSourcesUrlBase + "/build.gradle.kts");
        VERSION_SOURCES.put(Source.MAVEN, versionSourcesUrlBase + "/pom.xml");
        VERSION_SOURCES.put(Source.VERSION_FILE, versionSourcesUrlBase + "/VERSION");

        VersionSource vs = new VersionSource();

        for (Map.Entry<Source, String> entry : VERSION_SOURCES.entrySet()) {
            String content = getVersionIndicatorRaw(entry.getValue());
            if (content != null) {
                vs.content = content;
                vs.source = entry.getKey();
                break;
            }
        }

        return extractVersionFromSource(vs);
    }

    public static String getVersionFromGitCommit(String gitUrl, String org, String repo, String commit) {
        String versionSourcesUrlBase;
        if (org == null && repo == null) {
            if (gitUrl.endsWith(".git")) {
                versionSourcesUrlBase = gitUrl.substring(0, gitUrl.length() - 4) + "/raw/commit/" + commit;
            } else {
                versionSourcesUrlBase = gitUrl + "/raw/branch/" + commit;
            }
            if (!gitUrl.startsWith("https://")) {
                versionSourcesUrlBase = "https://" + versionSourcesUrlBase;
            }
        } else {
            versionSourcesUrlBase = "https://" + gitUrl + "/" + org + "/" + repo + "/raw/commit/" + commit;
        }
        Map<Source, String> VERSION_SOURCES = new HashMap<>();
        VERSION_SOURCES.put(Source.GRADLE_GROOVY, versionSourcesUrlBase + "/build.gradle");
        VERSION_SOURCES.put(Source.GRADLE_KOTLIN,versionSourcesUrlBase + "/build.gradle.kts");
        VERSION_SOURCES.put(Source.MAVEN, versionSourcesUrlBase + "/pom.xml");
        VERSION_SOURCES.put(Source.VERSION_FILE, versionSourcesUrlBase + "/VERSION");

        VersionSource vs = new VersionSource();

        for (Map.Entry<Source, String> entry : VERSION_SOURCES.entrySet()) {
            String content = getVersionIndicatorRaw(entry.getValue());
            if (content != null) {
                vs.content = content;
                vs.source = entry.getKey();
                break;
            }
        }

        return extractVersionFromSource(vs);
    }

    public static String getLatestBuildStatusFromJenkins(String jenkinsUrl, String projectName) {
        return getJenkinsProjectJson(jenkinsUrl, projectName).get("result").getAsString();
    }

    public static String getLatestSuccessfulVersionFromJenkins(String jenkinsUrl, String projectName) {
        JsonObject json = getJenkinsProjectJson(jenkinsUrl, projectName);
        String gitUrl = json.get("actions").getAsJsonArray().get(2).getAsJsonObject().get("remoteUrls").getAsJsonArray().get(0).getAsString();
        gitUrl = gitUrl.substring(0, gitUrl.length() - 4);
        String commit = json.get("changeSet").getAsJsonObject().get("items").getAsJsonArray().get(0).getAsJsonObject().get("commitId").getAsString();

        return getVersionFromGitCommit(gitUrl, null, null, commit);
    }

    private static JsonObject getJenkinsProjectJson(String jenkinsUrl, String projectName) {
        HttpClient client = HttpClient.newHttpClient();
        HttpRequest request = HttpRequest.newBuilder()
                .GET()
                .header("accept", "application/json")
                .uri(URI.create("https://" + jenkinsUrl + "/job/" + projectName + "/lastSuccessfulBuild/api/json"))
                .build();
        String responseString = "";
        try {
            HttpResponse<String> response = client.send(request, HttpResponse.BodyHandlers.ofString());
            responseString = response.body();
        } catch (IOException | InterruptedException e) {
            e.printStackTrace();
        }

        return new Gson().fromJson(responseString, JsonObject.class);
    }

    private static String getVersionIndicatorRaw(String url) {
        HttpClient gitClient = HttpClient.newHttpClient();
        HttpRequest gitRequest = HttpRequest.newBuilder()
                .GET()
                .header("accept", "text/plain")
                .uri(URI.create(url))
                .build();
        try {
            HttpResponse<String> response = gitClient.send(gitRequest, HttpResponse.BodyHandlers.ofString());
            return response.body();
        } catch (final Exception e) {
            return null;
        }
    }

    private static String extractVersionFromSource(VersionSource vs) {
        String[] split = vs.content.split("\n");
        switch (vs.source) {
            case GRADLE_GROOVY -> {
                for (String str : split) {
                    if (str.startsWith("version")) {
                        return str.substring(9, str.length() - 1);
                    }
                }
            }
            case GRADLE_KOTLIN -> {
                for (String str : split) {
                    if (str.startsWith("version =")) {
                        return str.substring(10, str.length() - 1);
                    }
                }
            }
            case MAVEN -> {
                for (String str : split) {
                    if (str.startsWith("  <version>")) {
                        return str.substring(11, str.length() - 1);
                    }
                }
            }
            case VERSION_FILE ->  {
                return vs.content;
            }
        }
        return "unknown";
    }

    enum Source {
        GRADLE_GROOVY,
        GRADLE_KOTLIN,
        MAVEN,
        VERSION_FILE
    }

    private static class VersionSource {
        private Source source;
        private String content;
    }
}
